unit XwDMU;

interface

uses
  XwType,
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Db, DBTables;

type
  TXwDM = class(TDataModule)
    Database: TDatabase;
    XwT: TTable;
    XwS: TDataSource;
    XwTXWord: TStringField;
    XwTXWExplane: TMemoField;
    XwTXWUsed: TSmallintField;
    XwTXW1: TStringField;
    XwTXW2: TStringField;
    XwTXW3: TStringField;
    XwTXW4: TStringField;
    XwTXW5: TStringField;
    procedure XwDMCreate(Sender: TObject);
    procedure XwDMDestroy(Sender: TObject);
  private
     FAliasPath : string;
     FXwDBFileName : string;

     procedure SetAliasPath( Value : string);
     procedure SetXwDBFileName( Value : string);
  public
    property AliasPath : string read FAliasPath write SetAliasPath;
    property XwDBFileName : string read FXwDBFileName write SetXwDBFileName;
    procedure CreateXwTable( FName : string);
    function AddWordToXwDB( aWord : string) : boolean;
  end;

var
  XwDM: TXwDM;

implementation

{$R *.DFM}

procedure TXwDM.XwDMCreate(Sender: TObject);
begin
  AliasPath := ExtractFileDir( ParamStr( 0)) + '\';
end;

procedure TXwDM.XwDMDestroy(Sender: TObject);
begin
  FXwDBFileName := '';
  Database.Connected := false;
end;

procedure TXwDM.SetAliasPath( Value : string);
begin
  with Database do begin
    Connected := false;
    Params.Clear;
    Params.Add('PATH=' + Value);
    Connected := true;
    FAliasPath := Value;
    end;
end;

procedure TXwDM.SetXwDBFileName( Value : string);
begin
  AliasPath := ExtractFilePath( Value);
  XwT.TableName := ExtractFileName( Value);
  try
    XwT.Open;
    FXwDBFileName := XwT.TableName;
  except
    FXwDBFileName := '';
    raise Exception.Create('Not XWord database or file corrupted');
    end;
end;

procedure TXwDM.CreateXwTable( FName : string);
var Table : TTable;  Cou : integer;
begin
  AliasPath := ExtractFilePath( FName);

  Table := TTable.Create( NIL);
  with Table do begin
    try
      DatabaseName := Self.Database.DatabaseName;
      TableName := ExtractFileName( FName);
{ add fields}
FieldDefs.Add( 'XWord', ftString, MaxWordLen, false);  //word
FieldDefs.Add( 'XwExplane', ftMemo, 240, false); // word description
FieldDefs.Add( 'XwUsed', ftSmallint, 0, false); // word used count
for Cou := 1 to MaxWordIndexNum do
  FieldDefs.Add( FieldNameByIndex( Cou), ftString,
                 MaxWordLen - pred( Cou), false);
{ add indexes}
IndexDefs.Add( '', 'XWord', [ixPrimary]);
for Cou := 1 to MaxWordIndexNum do
  IndexDefs.Add( 'i' + FieldNameByIndex( Cou),
                 FieldNameByIndex( Cou), [ixCaseInsensitive]);
      CreateTable;
    finally
      Close;
      Free;
      end;
    end;
end;

function TXwDM.AddWordToXwDB( aWord : string) : boolean;
var Cou : integer;
begin
  Result := false;
  aWord := Trim( aWord);
  if ( aWord = '') OR ( length( aWord) < 3) then exit;
  with XwT do begin
    Append;
    XwTXWord.Value := AnsiUpperCase( aWord);
    XwTXWUsed.Value := 0;
    for Cou := 1 to MaxWordIndexNum do begin
      FieldByName( FieldNameByIndex( Cou)).AsString :=
        WordIDbyCharIndex( aWord, Cou);
      end;
    try
      Post;
      Result := true;
    except Cancel; end;
    end;
end;

end.
