unit XwDatabaseU;

interface

uses
  XwType, XwDMU,
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Grids, DBGrids, Buttons, ExtCtrls, Menus, DBCtrls, ComCtrls;

type
  TXwDatabaseF = class(TForm)
    OpenD: TOpenDialog;
    SaveD: TSaveDialog;
    Menu: TMainMenu;
    File1: TMenuItem;
    N1: TMenuItem;
    ExitMn: TMenuItem;
    OpenMn: TMenuItem;
    NewMn: TMenuItem;
    Panel2: TPanel;
    Panel3: TPanel;
    XwN: TDBNavigator;
    XwG: TDBGrid;
    Open1: TMenuItem;
    N2: TMenuItem;
    Panel4: TPanel;
    Panel5: TPanel;
    Memo: TMemo;
    AddDBBB: TBitBtn;
    Label1: TLabel;
    FileNameL: TLabel;
    ProgressB: TProgressBar;
    Label7: TLabel;
    FindL: TLabel;
    Label10: TLabel;
    AddL: TLabel;
    LoadSB: TSpeedButton;
    SpeedButton1: TSpeedButton;
    Label2: TLabel;
    DbFileNameL: TLabel;
    Label3: TLabel;
    DbRecCouL: TLabel;
    About1: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure OpenMnClick(Sender: TObject);
    procedure ExitMnClick(Sender: TObject);
    procedure OpenMnDBClick(Sender: TObject);
    procedure NewMnClick(Sender: TObject);
    procedure AddDBBBClick(Sender: TObject);
    procedure About1Click(Sender: TObject);
  protected
    FileName : string;
    FCanConvert : boolean;

    procedure SetCanConvert( Value : boolean);
  private
    property  CanConvert : boolean read FCanConvert write SetCanConvert;
    procedure AddToDB;
  public
    procedure LoadDB( FName : string);
    procedure CreateDB( FName : string);
    procedure LoadFile( FName : string);
  end;

var
  XwDatabaseF: TXwDatabaseF;

implementation
{$R *.DFM}
uses XwAboutWebU;

procedure TXwDatabaseF.FormCreate(Sender: TObject);
begin
  CanConvert := false;
  FileName := '';
  FileNameL.Caption :=  '';
  DbFileNameL.Caption := '';
end;

procedure TXwDatabaseF.FormDestroy(Sender: TObject);
begin
{}
end;

procedure TXwDatabaseF.LoadDB( FName : string);
var FN : string;
begin
  FN := '';
  if FName <> '' then FN := FName
  else begin
    OpenD.Title := 'Open XWord database';
    OpenD.Filter := 'XWord database|*.DB|All files|*.*';
    OpenD.FilterIndex := 0;
    if OpenD.Execute then FN := OpenD.FileName;
    end;
  if FN <> '' then begin
    try
      XwDM.XwDBFileName := FN;
      DbFileNameL.Caption := ExtractFileName( FN);
      DbRecCouL.Caption := IntToStr( XwDM.XwT.RecordCount);
    except
      DbFileNameL.Caption := '';
      DbRecCouL.Caption := '';
      raise;
      end;
    end;
  CanConvert := ( XwDM.XwDBFileName <> '') AND ( FileName <> '');
end;

procedure TXwDatabaseF.CreateDB( FName : string);
var FN : string;
begin
  OpenD.Title := 'Create XWord database';
  OpenD.Filter := 'XWord database|*.DB|All files|*.*';
  OpenD.FilterIndex := 0;
  OpenD.DefaultExt := 'db';
  if OpenD.Execute then begin
    try
      XwDM.CreateXwTable( OpenD.FileName);
      LoadDB( OpenD.FileName);
    except raise end;
    end;
  OpenD.DefaultExt := '';
end;

procedure TXwDatabaseF.LoadFile( FName : string);
begin
  FileName := '';
  Memo.Lines.Clear;
  if FName <> '' then FileName := FName
  else begin
    OpenD.FileName := '';
    OpenD.Title := 'Open text file';
    OpenD.Filter := 'Text|*.txt|Pascal source|*.pas|All files|*.*';
    OpenD.FilterIndex := 2;
    if OpenD.Execute then FileName := OpenD.FileName;
    end;
  if FileName <> '' then begin
    Memo.Lines.LoadFromFile( FileName);
    end;
  FileNameL.Caption := ExtractFileName( FileName);
  ProgressB.Position := 0;
  CanConvert := ( XwDM.XwDBFileName <> '') AND ( FileName <> '');
end;

procedure TXwDatabaseF.SetCanConvert( Value : boolean);
begin
end;

procedure TXwDatabaseF.OpenMnClick(Sender: TObject);
begin
  LoadFile( '');
end;

procedure TXwDatabaseF.ExitMnClick(Sender: TObject);
begin
  Close;
end;

procedure TXwDatabaseF.OpenMnDBClick(Sender: TObject);
begin
  LoadDB( '');
end;

procedure TXwDatabaseF.AddToDB;
var
  Parser : TParser;
  FStr : TFileStream;
  Symb : string;
  TokFind, TokAdd : integer;

  procedure ShowParseVal;
  var S : string;
  begin
    with Parser do begin
      case Token of
        toEOF:     S := 'toEOF';
        toSymbol:  S := 'toSymbol';
        toString:  S := 'toString';
        toInteger: S := 'toInteger';
        toFloat:   S := 'toFloat';
        end;
      ProgressB.Position := SourcePos;
      FindL.Caption := IntToStr( TokFind);
      AddL.Caption := IntToStr( TokAdd);
      end;
  end;

begin
  if XwDM.XwDBFileName = '' then
    raise Exception.Create( 'Load XWord database or create new');
  if FileName = '' then
    raise Exception.Create( 'Load text file');

  TokFind := 0; TokAdd := 0;
  FStr := TFileStream.Create( FileName, fmOpenRead);
  ProgressB.Max := FStr.Size; ProgressB.Position := 0;

  Parser := TParser.Create( FStr);
  XwDM.XwT.DisableControls;

  with Parser do begin
    while Token <> toEOF do begin
      Application.ProcessMessages;
      ShowParseVal;
      if TokenSymbolIs( TokenString) then begin
        inc( TokFind);
        try
          Symb := TokenString;
          if XwDM.AddWordToXwDB( Symb) then inc( TokAdd);
        except Symb := ''; end;
        end;
      try NextToken;
      except end;
      end;
    end;

  DbRecCouL.Caption := IntToStr( XwDM.XwT.RecordCount);
  XwDM.XwT.EnableControls;
  Parser.Free;
  FStr.Free;
end;


procedure TXwDatabaseF.NewMnClick(Sender: TObject);
begin
  CreateDB( '');
end;

procedure TXwDatabaseF.AddDBBBClick(Sender: TObject);
begin
  AddToDB;
end;

procedure TXwDatabaseF.About1Click(Sender: TObject);
begin
  XwAboutWebF.ShowModal;
end;

end.
