unit XwdU;

interface

uses
  XWType, XWObj, XWCont,
  ShellAPI,
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, Menus, ExtCtrls, Buttons;

const
  AppVersion = '1.12';
  
  { Stream ID and Settings length}
  sidXWordDesktop    = $F001; slXWordDesktop = 100;

type
  TXWordF = class(TForm)
    StatusB: TStatusBar;
    MainM: TMainMenu;
    File1: TMenuItem;
    Help1: TMenuItem;
    Open1: TMenuItem;
    Save1: TMenuItem;
    N1: TMenuItem;
    Exit1: TMenuItem;
    Panel1: TPanel;
    XWControlsP: TPanel;
    XWordSB: TScrollBox;
    OpenD: TOpenDialog;
    WEditP: TPanel;
    WDescP: TPanel;
    Splitter1: TSplitter;
    ToolsP: TPanel;
    LoadSB: TSpeedButton;
    SaveSB: TSpeedButton;
    OptPannelsSB: TSpeedButton;
    Options1: TMenuItem;
    Panel2: TMenuItem;
    SaveAs1: TMenuItem;
    Buttonbar1: TMenuItem;
    Application1: TMenuItem;
    WebLoadSB: TSpeedButton;
    OpenfromWeb1: TMenuItem;
    SaveD: TSaveDialog;
    XwCheckCB: TSpeedButton;
    Webpage1: TMenuItem;
    XwPanelPM: TPopupMenu;
    Hint1: TMenuItem;
    N2: TMenuItem;
    Hint2: TMenuItem;
    HintSB: TSpeedButton;
    procedure Exit1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Splitter1Moved(Sender: TObject);
    procedure Save1Click(Sender: TObject);
    procedure Open1Click(Sender: TObject);
    procedure Buttonbar1Click(Sender: TObject);
    procedure SaveSBClick(Sender: TObject);
    procedure Panel2Click(Sender: TObject);
    procedure SaveAs1Click(Sender: TObject);
    procedure OpenfromWeb1Click(Sender: TObject);
    procedure XwCheckCBClick(Sender: TObject);
    procedure Application1Click(Sender: TObject);
    procedure Webpage1Click(Sender: TObject);
    procedure Hint1Click(Sender: TObject);
    procedure Hint2Click(Sender: TObject);
    procedure HintSBClick(Sender: TObject);
  private
    XPanel : TXWPanel;
    WEdit : TXWordEdit;
    WDesc : TXWDescGrid;
    XPContainer : TXWDControlCollection;

    FXWLoaded : boolean;
    FXWChanged : boolean;
    FXWFileName : string;
    ProgramDir : string;

    procedure LoadPattern( FName : string);
    function LoadFromEncriptStream( Stream : TStream) : boolean;
    procedure SavePattern( FName : string);

    procedure SaveSettings( Stream: TStream);
    procedure LoadSettings( Stream: TStream);

    procedure WordChanged(Sender: TObject);
    procedure XwHint;

    procedure SetXWLoaded( Value : boolean);
    procedure SetXWChanged( Value : boolean);
    procedure SetXWFileName( Value : string);
    procedure CalcStatistics;

  protected
    procedure WMDropFiles(var Msg: TMessage); message wm_DropFiles;

    property XWLoaded : boolean read FXWLoaded write SetXWLoaded;
    property XWChanged : boolean read FXWChanged write SetXWChanged;
    property XWFileName : string read FXWFileName write SetXWFileName;
  public
    { Public declarations }
  end;

var
  XWordF: TXWordF;

implementation
{$R *.DFM}
uses XwOptPanelU,
{$IFDEF USEIMS}
  WebDownloadU,
{$ENDIF}
  XwAboutU, XwAboutWebU, HintU;

procedure TXWordF.FormCreate(Sender: TObject);
begin
{$IFDEF USEIMS}
  WebLoadSB.Enabled := true;
  OpenfromWeb1.Enabled := true;
{$ENDIF}

  DragAcceptFiles(Handle, True);

  XPanel := TXWPanel.Create( XWordSB);
  with XPanel do begin
    Parent := XWordSB;
    Width := 470;
    Height := 470;
    CellSize := 20;
    PopupMenu := XwPanelPM;
    end;

  WEdit := TXWordEdit.Create( WEditP);
  with WEdit do begin
    Parent := WEditP;
    Top := 2;
    Left := 2;
    Font.Name := 'Courier New';
    Font.Size := 12;
    Width := WEditP.Width - 4;
    WEditP.Height := Height + 4;
    end;

  WDesc := TXWDescGrid.Create( WDescP);
  with WDesc do begin
    Parent := WDescP;
//    Align := alNone;
//    Height := 200;
    Align := alClient;
    ColWidths[ 1] := 99;
    ColWidths[ 2] := WDescP.Width - ColWidths[ 0] - ColWidths[ 1] - 21;
    end;

  XPContainer := TXWDControlCollection.Create;
  with XPContainer do begin
    OnWordChange := WordChanged;
    XPContainer.XWPanel := XPanel;
    XPContainer.XWordEdit := WEdit;
    XPContainer.XWDescGrid := WDesc;
    end;

  FXWLoaded := true; XWLoaded := false;
  FXWChanged := true; XWChanged := false;

  ProgramDir := ExtractFilePath(ParamStr(0));
  OpenD.InitialDir := ProgramDir;
  SaveD.InitialDir := ProgramDir;
  // check if we should load a file from the command line
  if (ParamCount > 0) and FileExists(ParamStr(1)) then
    LoadPattern( ParamStr(1));
end;

procedure TXWordF.FormDestroy(Sender: TObject);
begin
  XPContainer.Free;
  WDesc.Free;
  WEdit.Free;
  XPanel.Free;
  DragAcceptFiles(Handle, False);
end;

{- Handle wm_DropFiles message }
procedure TXWordF.WMDropFiles(var Msg: TMessage);
var
  Filename: array[0 .. 256] of Char;
begin
  DragQueryFile(
    THandle(Msg.WParam),
    0,
    Filename,
    SizeOf(Filename));
  LoadPattern( StrPas(FileName));
  DragFinish( THandle(Msg.WParam));
end;


procedure TXWordF.CalcStatistics;
var
  WComplete, WPartialFill, WEmpty, WTotal : integer;
begin
  WTotal := XPContainer.Count;
  XPContainer.XWStat( WComplete, WPartialFill, WEmpty);
  XwCheckCB.Enabled := WTotal = WComplete;

  StatusB.Panels[1].Text :=
    'Filled ' + IntToStr( WComplete) + ' of ' + IntToStr( WTotal);
end;

procedure TXWordF.WordChanged( Sender: TObject);
begin
  XWChanged := true;
  CalcStatistics;
end;

procedure TXWordF.Exit1Click(Sender: TObject);
begin Close; end;

procedure TXWordF.SaveSettings( Stream: TStream);
var
  LenCou, TmpInt : integer;
//  PanColor : TColor;
  Buf : array[1..slXWordDesktop] of byte;
begin
  LenCou := 0;

  //Signature and length
  TmpInt := sidXWordDesktop;
  Stream.WriteBuffer( TmpInt, SizeOf( TmpInt));
  TmpInt := slXWordDesktop;
  Stream.WriteBuffer( TmpInt, SizeOf( TmpInt));

  TmpInt := ClientWidth;
  Stream.WriteBuffer( TmpInt, SizeOf( TmpInt)); inc( LenCou, SizeOf( TmpInt));
  TmpInt := ClientHeight;
  Stream.WriteBuffer( TmpInt, SizeOf( TmpInt)); inc( LenCou, SizeOf( TmpInt));
  TmpInt := Top;
  Stream.WriteBuffer( TmpInt, SizeOf( TmpInt)); inc( LenCou, SizeOf( TmpInt));
  TmpInt := Left;
  Stream.WriteBuffer( TmpInt, SizeOf( TmpInt)); inc( LenCou, SizeOf( TmpInt));

  TmpInt := XWControlsP.Width;
  Stream.WriteBuffer( TmpInt, SizeOf( TmpInt)); inc( LenCou, SizeOf( TmpInt));

  FillChar( Buf, slXWordDesktop, #0);
  Stream.WriteBuffer( Buf, slXWordDesktop - LenCou);
end;

procedure TXWordF.LoadSettings( Stream: TStream);
var
  LenCou, TmpInt : integer;
//  PanColor : TColor;
  Buf : array[1..slXWordDesktop] of byte;
begin
  LenCou := 0;

  // Read signature and process
  Stream.ReadBuffer( TmpInt, SizeOf( TmpInt));
  if TmpInt <> sidXWordDesktop then begin
    Stream.Position := Stream.Position - SizeOf( TmpInt);
    exit;
    end;
  // Read data length ( not used)
  Stream.ReadBuffer( TmpInt, SizeOf( TmpInt));

  Stream.ReadBuffer( TmpInt, SizeOf( TmpInt)); inc( LenCou, SizeOf( TmpInt));
  ClientWidth := TmpInt;
  Stream.ReadBuffer( TmpInt, SizeOf( TmpInt)); inc( LenCou, SizeOf( TmpInt));
  ClientHeight := TmpInt;
  Stream.ReadBuffer( TmpInt, SizeOf( TmpInt)); inc( LenCou, SizeOf( TmpInt));
  Top := TmpInt;
  Stream.ReadBuffer( TmpInt, SizeOf( TmpInt)); inc( LenCou, SizeOf( TmpInt));
  Left := TmpInt;

  Stream.ReadBuffer( TmpInt, SizeOf( TmpInt)); inc( LenCou, SizeOf( TmpInt));
  XWControlsP.Width := TmpInt;
  Splitter1Moved( Splitter1);

  Stream.ReadBuffer( Buf, slXWordDesktop - LenCou);
end;

function TXWordF.LoadFromEncriptStream( Stream : TStream) : boolean;
var
  EStre : TWxEncriptionStream;
  LoadCou : integer;
begin
  EStre := TWxEncriptionStream.Create;
  EStre.CopyFrom( Stream, Stream.Size);

  EStre.Decript;
  EStre.Position := 0;

  LoadCou := 0;
  try
    while ( EStre.Position <= EStre.Size - 1) AND ( LoadCou < 10) do begin
      LoadSettings( EStre);
      XPanel.LoadSettings( EStre);
      WEdit.LoadSettings( EStre);
      WDesc.LoadSettings( EStre);
      XPContainer.LoadFromStream( EStre);
      inc( LoadCou);
      end;
    Result := LoadCou < 10;
  except Result := false; end;

  EStre.Free;
end;

procedure TXWordF.LoadPattern( FName : string);
var
  FN, FExt : string;
  Loaded : boolean;

  function LoadTextPattern( FN : string) : boolean;
  begin
    XPContainer.LoadFromTextFile( FN);
    Result := true;
  end;

  function LoadCompressPattern( FN : string) : boolean;
  var
    Stre : TFileStream;
  begin
    Stre := TFileStream.Create( FN, fmOpenRead);
    Result := LoadFromEncriptStream( Stre);
    Stre.Free;
  end;

  function LoadDesktop( FN : string) : boolean;
  var
    Stre : TFileStream;
  begin
    Stre := TFileStream.Create( FN, fmOpenRead);
    Result := LoadFromEncriptStream( Stre);
    Stre.Free;
  end;

begin
  FN := '';
  Loaded := false;
  if FName <> '' then FN := FName
  else begin
    OpenD.Title := 'Open XWord file';
{    OpenD.FileName := '*' +  extXWPattern +
                      ';*' + extComprXWPattern +
                      ';*' + extXWDesktop;}
    if OpenD.Execute then FN := OpenD.FileName;
    end;
  if FN <> '' then begin
    FExt := AnsiLowerCase( ExtractFileExt( FN));
    if FExt = AnsiLowerCase( extXWPattern) then
      Loaded := LoadTextPattern( FN) else
    if FExt = AnsiLowerCase( extComprXWPattern) then
      Loaded := LoadCompressPattern( FN) else
    if FExt = AnsiLowerCase( extXWDesktop) then
      Loaded := LoadDesktop( FN);
    if Loaded then begin
      try
        XPContainer.Check;
        XWLoaded := true;
        XWFileName := FN;
      except
        XWLoaded := false;
        raise;
        end;
      end;
    end;
end;

procedure TXWordF.SavePattern( FName : string);

  procedure SaveDesktop( FName : string);
  var
    Stre : TFileStream;
    EStre : TWxEncriptionStream;
    FP : integer;
    FE : string;
  begin
    FE := ExtractFileExt( FName);
    FP := System.Pos( FE, FName);
    System.Delete( FName, FP, length( FE));
    FName := FName + extXWDesktop;

    EStre := TWxEncriptionStream.Create;
    Stre := TFileStream.Create( FName, fmCreate);

    SaveSettings( EStre);
    XPanel.SaveSettings( EStre);
    WEdit.SaveSettings( EStre);
    WDesc.SaveSettings( EStre);
    XPContainer.SaveToStream( EStre); // pattern data

    EStre.Encript;
    EStre.Position := 0;
    Stre.CopyFrom( EStre, EStre.Size);

    Stre.Free;
    EStre.Free;
  end;

begin
  if FName <> '' then begin
    SaveD.Title := 'Save';
    SaveDesktop( FName);
    end
  else
    with SaveD do begin
      Title := 'Save as ...';
      FileName := '*' + extXWDesktop;
      if Execute then begin
        SaveDesktop( FileName);
        end;
      end;
  XWChanged := false;    
end;

procedure TXWordF.Splitter1Moved(Sender: TObject);
begin
  with WEdit do begin
    WEdit.Width := WEditP.Width - 4;
    end;
  with WDesc do begin
    ColWidths[ 2] := WDescP.Width - ColWidths[ 0] - ColWidths[ 1] - 21;
    end;
end;

procedure TXWordF.Save1Click(Sender: TObject);
begin SavePattern( XWFileName); end;

procedure TXWordF.Open1Click(Sender: TObject);
begin
  LoadPattern( '');
end;

procedure TXWordF.Buttonbar1Click(Sender: TObject);
begin
  ( Sender as TMenuItem).Checked := NOT ( Sender as TMenuItem).Checked;
  ToolsP.Visible := ( Sender as TMenuItem).Checked;
end;

procedure TXWordF.SaveSBClick(Sender: TObject);
begin SavePattern( XWFileName) end;

procedure TXWordF.Panel2Click(Sender: TObject);
begin
  XwOptPanelF.SetSettings( XPanel, WEdit, WDesc);
  if XwOptPanelF.ShowModal = mrOk then begin
    XwOptPanelF.AssignSettings( XPanel, WEdit, WDesc);
    WEdit.Width := WEditP.Width - 4;
    WEditP.Height := WEdit.Height + 4;
    XWChanged := true;
    end;
end;

procedure TXWordF.SetXWLoaded( Value : boolean);
begin
  if FXWLoaded = Value then exit;
  FXWLoaded := Value;
  SaveSB.Enabled := FXWLoaded;
  Save1.Enabled := FXWLoaded;
  SaveAs1.Enabled := FXWLoaded;
  Hint1.Enabled := FXWLoaded;
  Hint2.Enabled := FXWLoaded;
  HintSB.Enabled := FXWLoaded;
  Panel2.Enabled := FXWLoaded;
  OptPannelsSB.Enabled := FXWLoaded;
  XPanel.Visible := FXWLoaded;
  WEdit.Visible := FXWLoaded;
  WDesc.Visible := FXWLoaded;
  if FXWLoaded then begin
    XWChanged := true; XWChanged := false;
    CalcStatistics;
    end;
end;

procedure TXWordF.SetXWChanged( Value : boolean);
begin
  if FXWChanged = Value then exit;
  FXWChanged := Value;
  SaveSB.Enabled := FXWChanged;
  Save1.Enabled := FXWChanged;
end;

procedure TXWordF.SetXWFileName( Value : string);
begin
 FXWFileName :=  Value;
 StatusB.Panels[0].text := ExtractFileName( FXWFileName);
end;

procedure TXWordF.SaveAs1Click(Sender: TObject);
begin
  SaveD.Options := SaveD.Options + [ ofOverwritePrompt];
  SavePattern( '');
  SaveD.Options := SaveD.Options - [ ofOverwritePrompt];
end;

procedure TXWordF.OpenfromWeb1Click(Sender: TObject);
begin
{$IFDEF USEIMS}
  WebDownloadF := TWebDownloadF.Create( Self);
  try
    if WebDownloadF.ShowModal = mrOk then begin
      XWLoaded := LoadFromEncriptStream( WebDownloadF.XwStream);
      if XWLoaded then XWFileName := WebDownloadF.FileName;
      end;
  finally
    WebDownloadF.Free;
    end;
{$ELSE}
{$ENDIF}
end;

procedure TXWordF.XwCheckCBClick(Sender: TObject);
var
  WComplete, WPartialFill, WEmpty, WTotal, WMatch, Cou : integer;
  S : string;
begin
  WTotal := XPContainer.Count;
  XPContainer.XWStat( WComplete, WPartialFill, WEmpty);
  WMatch := 0;
  for Cou := 0 to WTotal-1 do begin
    if XPContainer.XMatch[ Cou] then inc( WMatch);
    end;
  S := Format( '%d words match out of %d', [ WMatch, WTotal]);
  MessageDlg( S, mtInformation, [ mbOK], 0);
end;

procedure TXWordF.Application1Click(Sender: TObject);
begin XwAboutF.ShowModal; end;

procedure TXWordF.Webpage1Click(Sender: TObject);
begin XwAboutWebF.ShowModal; end;

procedure TXWordF.XwHint;
begin
  with HintF do begin
    XwHint := XPContainer.XWItems[ XPContainer.ItemIndex].XEtalon;
    ShowModal;
    end;
end;

procedure TXWordF.Hint1Click(Sender: TObject);
begin XwHint; end;

procedure TXWordF.Hint2Click(Sender: TObject);
begin XwHint; end;

procedure TXWordF.HintSBClick(Sender: TObject);
begin XwHint; end;

end.
